<?php
/**
 * @author DWand
 * @version 13.10.2012
 * @package DSiteStructScanner
 *
 * @description Класс для сканировния структуры каталогов.
 *              Позволяет построить эталонную структуру каталогов и впоследствии
 *              сравнивать текущую структуру каталогов с эталонной.
 * 
 * Примечание:
 *    Структура файловой системы представляется в виде:
 *    array (
 *       'названиеЭлемента' => array (
 *          [0] => типЭлемента,
 *          [1] => информацияОбЭлементе
 *       ),
 *       ...
 *    )
 *    Где 
 *       типЭлемента          - указывает чем является элемент: файл (f) или каталог (dir)
 *       информацияОбЭлементе - отображает информацию об элементе в зависимости от типа:
 *          каталог - указвается массив вложенных элементов в общем формате (см. пример выше)
 *          файл - array (
 *                    [0] => размерФайла,
 *                    [1] => датаМодификацииФайла,
 *                    [2] => useridВладельцаФайла,
 *                    [3] => groupIdВладельцаФайла
 *                 )
 */

class DSiteStructScanner {
    /**
     * Произошедшие в ходе проверки ошибки
     * @var array(array(string[путь к файлу], string[описание ошибки]))
     */
    protected $erOcur;

    
    /**
     * Описание возможных ошибок
     */
    const ER_TYPE_NEED_DIR  = 'Неправильный тип элемента. Это файл, а ожидался каталог.';
    const ER_TYPE_NEED_FILE = 'Неправильный тип элемента. Это каталог, а ожидался файл.';
    const ER_NEW_FILE  = 'Этого файла раньше не было.';
    const ER_NEW_CAT   = 'Этого каталога раньше не было.';
    const ER_DIF_SIZE  = 'Изменился размер файла. [old] -> [new].';
    const ER_DIF_MOD   = 'Изменилось время модификаци файла. [old] -> [new].';
    const ER_DIF_USER  = 'Изменился userid владельца файла. [old] -> [new].';
    const ER_DIF_GROUP = 'Изменился groupid владельца файла. [old] -> [new].';
    const ER_NO_DIR    = 'Каталог не найден.';
    const ER_NO_FILE   = 'Файл не найден.';
    
    
    /**
     * Сканирует файловую систему на предмет сооветствия еталону
     * @param string $stDir Путь к начальной директория для сканирования
     * @param string $exNodes Массив путей к директориям, которые исключаются из сканирования
     * @param array(string, array(string, array...)) $etStr Массив с эталонной структурой
     * @return Boolean Соответствует ли текущая структура эталонной
     */
    public function scan($stDir, $exNodes, $etStr) {
        $trueStr = self::_buildStruct($stDir, $exNodes);
        return $this -> _compareStructs($trueStr, $etStr, $stDir);
    }
    
    
    /**
     * Возвращает информацию об ошибках
     * @return array(array(string[путь к файлу], string[описание ошибки]))
     */
    public function getErrors() {
        return $this -> erOcur;
    }
    
   
    /**
     * Возвращает информацию о произошедших ошибках в форматированном виде
     * @param string $erTpl Шаблон описания ошибки.<br />
     *                      Доступные подстановки:<br />
     *                      [path] - Путь к файлу<br />
     *                      [error] - Описание ошибки.
     * @param string $delim Разделитель между ошибками
     * @return string Отформатированная информация об ошибках
     */
    public function getFormatedErrors($erTpl = '[path] - [error]', $delim = '<br />') {
        $str = '';
        foreach ($this -> erOcur as $error) {
            $str .= str_replace(array('[path]', '[error]'), array($error[0], $error[1]), $erTpl) . $delim;
        }
        return $str;
    }
    
    
    /**
     * Считывает из файла эталонную файловую структуру и возвращает ее
     * @param string $etPath Путь к файлу с єталонной файловой структурой
     * @return array(string, array(string, array...))
     */
    static public function getEtalonFromStr($etStr) {
        return unserialize($etStr);
    }
    
    
    /**
     * Строит эталонную файловую структуру и возвращает ее в виде строки
     * @param type $stDir Начальная директория для сканирования
     * @param type $exNodes Каталоги, которые не нужно сканировать
     * @return string
     */
    static public function getEtalonStr($stDir, $exNodes) {
        return serialize(self::_buildStruct($stDir, $exNodes));
    }
    
    
    /**
     * Строит файловую структуру конкретного каталога
     * @param type $stDir Адрес целевого каталога
     * @param type $exNodes Каталоги, которые не нужно сканировать
     * @return array(string, array(string, array...))
     */
    static public function _buildStruct($stDir, $exNodes) {
        $inside = scandir($stDir);
        $struct = array();
        
        /**
         * Проходим все, что есть внутри каталога 
         */
        foreach ($inside as $node) {
            $path = $stDir. '/'. $node;

            /**
             * Проверяем, нужно ли сканировать этот узел 
             */
            if ($node == '.' || $node == '..' 
                || in_array($path, $exNodes)) {
                continue;
            }
            
            if (is_dir($path)) {
                /**
                 * Если текущий узел - это каталог, то 
                 * обозначаем, что это каталог и добавляем его структуру.
                 */
                $struct[$node] = array('d', self::_buildStruct($stDir . '/' . $node, $exNodes));
            } else {
                /**
                 * Если текущий узел - это не каталог, то 
                 * обозначаем, что это файл и добавляем информацию о нем.
                 */
                $stat = stat($path);
                $struct[$node] = array('f', array(
                    $stat['size'],
                    $stat['mtime'],
                    $stat['uid'],
                    $stat['gid']
                ));
            }
         }
         
         return $struct;
    }
    
    
    /**
     * Стравнивает две структуры.
     * В случае нахождения несоответствия добавляет информацию об ошибке.
     * @param array(string, array(string, array...)) $true Текущая структура
     * @param array(string, array(string, array...)) $etalon Эталонная структура
     * @param string $path Путь к сравниваемым структурам
     * @return Boolean Одинаковы ли данные структуры
     */
    protected function _compareStructs($true, $etalon, $path) {
        $wasErr = false;    // Возникали ли несоответствия
        
        foreach ($true as $name => $info) {
            $curPath = $path . '/' . $name;
            
            /**
             * Есть ли текущий элемент в эталонной структуре 
             */
            if (array_key_exists($name, $etalon)) {
                /**
                 * Совпадают ли типы элементов 
                 */
                if ($info[0] == $etalon[$name][0]) {
                    if ($info[0] == 'd' && !$this -> _compareStructs($info[1], $etalon[$name][1], $curPath)) {
                        /**
                         * Поскольку текущий элемент - каталог, то надо сравнить структуры каталогов.
                         * Если структуры отличаются, то данные про ошибку мы записали при сравнении.
                         * Просто обозначаем, что произошла ошибка.
                         */
                        $wasErr = true;
                    } else if ($info[0] == 'f') {
                        /**
                         * Текущий элемент - файл. Значит, сравниваем информацию о файле. 
                         */
                        
                        /**
                         * Сравниваем размер файла 
                         */
                        if ($info[1][0] != $etalon[$name][1][0]) {
                            $mes = str_replace(
                                    array('[old]', '[new]'), 
                                    array($etalon[$name][1][0], $info[1][0]),
                                    self::ER_DIF_SIZE);
                            $this -> erOcur[] = array($curPath, $mes);
                            $wasErr = true;
                        }
                        
                        /**
                         * Сравниваем дату модификации файла 
                         */
                        if ($info[1][1] != $etalon[$name][1][1]) {
                            $mes = str_replace(
                                    array('[old]', '[new]'), 
                                    array(date('r', $etalon[$name][1][1]), date('r', $info[1][1])),
                                    self::ER_DIF_MOD);
                            $this -> erOcur[] = array($curPath, $mes);
                            $wasErr = true;
                        }
                        
                        /**
                         * Сравниваем userid владельца файла
                         */
                        if ($info[1][2] != $etalon[$name][1][2]) {
                            $mes = str_replace(
                                    array('[old]', '[new]'), 
                                    array($etalon[$name][1][2], $info[1][2]),
                                    self::ER_DIF_USER);
                            $this -> erOcur[] = array($curPath, $mes);
                            $wasErr = true;
                        }
                        
                        /**
                         * Сравниваем groupid владельца файла
                         */
                        if ($info[1][3] != $etalon[$name][1][3]) {
                            $mes = str_replace(
                                    array('[old]', '[new]'), 
                                    array($etalon[$name][1][3], $info[1][3]),
                                    self::ER_DIF_GROUP);
                            $this -> erOcur[] = array($curPath, $mes);
                            $wasErr = true;
                        }
                    }
                } else {
                    /**
                     * Типы не совпадают (имхо, это, вообще, странно, но мало ли) 
                     */
                    if ($info[0] == 'd') $this -> erOcur[] = array($curPath, self::ER_TYPE_NEED_FILE);
                    else $this -> erOcur[] = array($curPath, self::ER_TYPE_NEED_DIR);
                }
                
                /**
                 * Убираем элемент из эталонной структуры, как обработанный.
                 */
                unset($etalon[$name]);
            } else {
                /**
                 * Элемента нет в эталонной структуре, значит появился новый. 
                 */
                $wasErr = true;
                if ($info[0] == 'f') $this -> erOcur[] = array($curPath, self::ER_NEW_FILE);
                else $this -> erOcur[] = array($curPath, self::ER_NEW_CAT);
            }
        }
        
        
        /**
         * Если структуры совпадают, то в эталонная структура должна быть пустой.
         * Иначе у нас не хватает файлов. То есть, какой-то файл удалили.
         */
        if (count($etalon) != 0) {
            $wasErr = true;
            foreach ($etalon as $name => $info) {
                $curPath = $path . '/' . $name;
                if ($info[0] == 'd') $this -> erOcur[] = array($curPath, self::ER_NO_DIR);
                else $this -> erOcur[] = array($curPath, self::ER_NO_FILE);
            }
        }
        
        return !$wasErr;
    }
}
?>